#
# The Python Imaging Library
# $Id$
#
# WCK-style drawing interface operations
#
# History:
# 2003-12-07 fl   created
# 2005-05-15 fl   updated; added to PIL as ImageDraw2
# 2005-05-15 fl   added text support
# 2005-05-20 fl   added arc/chord/pieslice support
#
# Copyright (c) 2003-2005 by Secret Labs AB
# Copyright (c) 2003-2005 by Fredrik Lundh
#
# See the README file for information on usage and redistribution.
#

from . import Image, ImageColor, ImageDraw, ImageFont, ImagePath


class Pen:
    def __init__(self, color, width=1, opacity=255):
        self.color = ImageColor.getrgb(color)
        self.width = width


class Brush:
    def __init__(self, color, opacity=255):
        self.color = ImageColor.getrgb(color)


class Font:
    def __init__(self, color, file, size=12):
        # FIXME: add support for bitmap fonts
        self.color = ImageColor.getrgb(color)
        self.font = ImageFont.truetype(file, size)


class Draw:
    def __init__(self, image, size=None, color=None):
        if not hasattr(image, "im"):
            image = Image.new(image, size, color)
        self.draw = ImageDraw.Draw(image)
        self.image = image
        self.transform = None

    def flush(self):
        return self.image

    def render(self, op, xy, pen, brush=None):
        # handle color arguments
        outline = fill = None
        width = 1
        if isinstance(pen, Pen):
            outline = pen.color
            width = pen.width
        elif isinstance(brush, Pen):
            outline = brush.color
            width = brush.width
        if isinstance(brush, Brush):
            fill = brush.color
        elif isinstance(pen, Brush):
            fill = pen.color
        # handle transformation
        if self.transform:
            xy = ImagePath.Path(xy)
            xy.transform(self.transform)
        # render the item
        if op == "line":
            self.draw.line(xy, fill=outline, width=width)
        else:
            getattr(self.draw, op)(xy, fill=fill, outline=outline)

    def settransform(self, offset):
        (xoffset, yoffset) = offset
        self.transform = (1, 0, xoffset, 0, 1, yoffset)

    def arc(self, xy, start, end, *options):
        self.render("arc", xy, start, end, *options)

    def chord(self, xy, start, end, *options):
        self.render("chord", xy, start, end, *options)

    def ellipse(self, xy, *options):
        self.render("ellipse", xy, *options)

    def line(self, xy, *options):
        self.render("line", xy, *options)

    def pieslice(self, xy, start, end, *options):
        self.render("pieslice", xy, start, end, *options)

    def polygon(self, xy, *options):
        self.render("polygon", xy, *options)

    def rectangle(self, xy, *options):
        self.render("rectangle", xy, *options)

    def text(self, xy, text, font):
        if self.transform:
            xy = ImagePath.Path(xy)
            xy.transform(self.transform)
        self.draw.text(xy, text, font=font.font, fill=font.color)

    def textsize(self, text, font):
        return self.draw.textsize(text, font=font.font)



#!/usr/bin/env python3

import getopt
import os
import shutil
import subprocess
import sys

from config import (
    VIRT_BASE,
    X64_EXT,
    bit_from_env,
    compiler_from_env,
    compilers,
    pythons,
    pyversion_from_env,
)


def setup_vms():
    ret = []
    for py in pythons:
        for arch in ("", X64_EXT):
            ret.append(
                "virtualenv -p c:/Python%s%s/python.exe --clear %s%s%s"
                % (py, arch, VIRT_BASE, py, arch)
            )
            ret.append(
                r"%s%s%s\Scripts\pip.exe install pytest pytest-cov"
                % (VIRT_BASE, py, arch)
            )
    return "\n".join(ret)


def run_script(params):
    (version, script) = params
    try:
        print("Running %s" % version)
        filename = "build_pillow_%s.cmd" % version
        with open(filename, "w") as f:
            f.write(script)

        command = ["powershell", "./%s" % filename]
        proc = subprocess.Popen(
            command,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        (trace, stderr) = proc.communicate()
        status = proc.returncode
        print("-- stderr --")
        print(stderr.decode())
        print("-- stdout --")
        print(trace.decode())
        print("Done with {}: {}".format(version, status))
        return (version, status, trace, stderr)
    except Exception as msg:
        print("Error with {}: {}".format(version, str(msg)))
        return (version, -1, "", str(msg))


def header(op):
    return r"""
setlocal
set MPLSRC=%%~dp0\..
set INCLIB=%%~dp0\depends
set BLDOPT=%s
cd /D %%MPLSRC%%
""" % (
        op
    )


def footer():
    return """endlocal
exit
"""


def vc_setup(compiler, bit):
    script = ""
    if compiler["vc_version"] == "2015":
        arch = "x86" if bit == 32 else "x86_amd64"
        script = (
            r"""
call "C:\Program Files (x86)\Microsoft Visual Studio 14.0\VC\vcvarsall.bat" %s
echo on"""
            % arch
        )
    return script


def build_one(py_ver, compiler, bit):
    # UNDONE virtual envs if we're not running on AppVeyor
    args = {}
    args.update(compiler)
    if "PYTHON" in os.environ:
        args["python_path"] = "%PYTHON%"
    else:
        args["python_path"] = "{}{}\\Scripts".format(VIRT_BASE, py_ver)

    args["executable"] = "python.exe"
    if "EXECUTABLE" in os.environ:
        args["executable"] = "%EXECUTABLE%"

    args["py_ver"] = py_ver
    args["tcl_ver"] = "86"

    if compiler["vc_version"] == "2015":
        args["imaging_libs"] = " build_ext --add-imaging-libs=msvcrt"
    else:
        args["imaging_libs"] = ""

    args["vc_setup"] = vc_setup(compiler, bit)

    script = r"""
setlocal EnableDelayedExpansion
call "%%ProgramFiles%%\Microsoft SDKs\Windows\%(env_version)s\Bin\SetEnv.Cmd" /Release %(env_flags)s
set DISTUTILS_USE_SDK=1
set LIB=%%LIB%%;%%INCLIB%%\%(inc_dir)s
set INCLUDE=%%INCLUDE%%;%%INCLIB%%\%(inc_dir)s;%%INCLIB%%\tcl%(tcl_ver)s\include

setlocal
set LIB=%%LIB%%;C:\Python%(py_ver)s\tcl%(vc_setup)s
call %(python_path)s\%(executable)s setup.py %(imaging_libs)s %%BLDOPT%%
call %(python_path)s\%(executable)s -c "from PIL import _webp;import os, shutil;shutil.copy(r'%%INCLIB%%\freetype.dll', os.path.dirname(_webp.__file__));"
endlocal

endlocal
"""  # noqa: E501
    return script % args


def clean():
    try:
        shutil.rmtree("../build")
    except Exception:
        # could already be removed
        pass
    run_script(("virtualenvs", setup_vms()))


def main(op):
    scripts = []

    for py_version, py_info in pythons.items():
        py_compilers = compilers[py_info["compiler"]][py_info["vc"]]
        scripts.append(
            (
                py_version,
                "\n".join(
                    [header(op), build_one(py_version, py_compilers[32], 32), footer()]
                ),
            )
        )

        scripts.append(
            (
                "{}{}".format(py_version, X64_EXT),
                "\n".join(
                    [
                        header(op),
                        build_one("%sx64" % py_version, py_compilers[64], 64),
                        footer(),
                    ]
                ),
            )
        )

    results = map(run_script, scripts)

    for (version, status, trace, err) in results:
        print("Compiled {}: {}".format(version, status and "ERR" or "OK"))


def run_one(op):

    compiler = compiler_from_env()
    py_version = pyversion_from_env()
    bit = bit_from_env()

    run_script(
        (
            py_version,
            "\n".join([header(op), build_one(py_version, compiler, bit), footer()]),
        )
    )


if __name__ == "__main__":
    opts, args = getopt.getopt(sys.argv[1:], "", ["clean", "wheel"])
    opts = dict(opts)

    if "--clean" in opts:
        clean()

    op = "install"
    if "--wheel" in opts:
        op = "bdist_wheel"

    if "PYTHON" in os.environ:
        run_one(op)
    else:
        main(op)



from PIL import Image

im = Image.open("../imgs/space.jpeg")
im.load()
print("Size: {}x{}".format(*im.size))

out_size = (320, 213)

im.resize(out_size, Image.BICUBIC, reducing_gap=None).save("_bic.ref.png")
for reducing_gap in range(1, 4):
    im.resize(out_size, Image.BICUBIC, reducing_gap=reducing_gap).save(
        f"_bic.{reducing_gap}.png"
    )

im.resize(out_size, Image.LANCZOS, reducing_gap=None).save("_lzs.ref.png")
for reducing_gap in range(1, 4):
    im.resize(out_size, Image.LANCZOS, reducing_gap=reducing_gap).save(
        f"_lzs.{reducing_gap}.png"
    )



from functools import partial
from timeit import repeat

from PIL import Image


def run_tests(name, stmt):
    print()
    print(name)
    print("=" * len(name))
    print("   ", "".join(f"{x:>6}" for x in range(1, 16)))

    for yscale in range(1, 16):
        print(f"{yscale:>3}:", end="")
        for xscale in range(1, 16):
            to_run = partial(stmt, xscale, yscale)
            runs = sorted(repeat(to_run, number=1, repeat=7))
            median_ms = sum(runs[2:-2]) / (len(runs) - 4) * 1000
            print(f" {f'{median_ms:5.1f}':>5}", end="")
        print()


im = Image.open("../imgs/space.jpeg")
im.load()
print("Size: {}x{}".format(*im.size))


def reduce(xscale, yscale):
    return im.reduce((xscale, yscale))


def box_resize(xscale, yscale):
    size = (im.width // xscale, im.height // yscale)
    return im.resize(size, Image.BOX, (0, 0, size[0] * xscale, size[1] * yscale))


run_tests("Reduce", reduce)
# run_tests("BOX resize", box_resize)
